/****************************************************************************************** 
 *	Chili DirectX Framework Version 12.04.24											  *	
 *	Game.h																				  *
 *	Copyright 2012 PlanetChili.net														  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#pragma once

#include "D3DGraphics.h"
#include "Keyboard.h"
#include "Mouse.h"
#include "Sound.h"
#define WORLDWIDTH 4000
#define WORLDHEIGHT 1199
// resolution of background grid
#define GRIDRESOLUTION 50
// camera tracking offset vertical/horizontal
// is the offset from the edge off the screen 
// to the camera tracking rectangle
#define CTRKOFFVERT 50
#define CTRKOFFHOR 240
#define NPLATFORMS 10
#define NENEMYS 5
#define NBUTTONS 4

class Game
{
	struct Menu
	{
		int r;
		int g;
		int b;
	};
	struct Player
	{
		float x;
		float y;
		float hp;
		float jump; 
		int jumpHeight; 
		int bulletX;
		int bulletY;
		bool isPlayerDead;
		bool TouchingBottom;
		bool OnPlatform;
		bool OnGround;
		bool canJump; 
		bool canShoot; 
		bool keyPressedLastFrame;
	};
	struct Enemy
	{
		float x;
		float y;
		float hp;
		bool isDead;
	};
	struct Platform
	{
		int x;
		int y; 
		int width;
		int height; 
		 
	};
	struct Button
	{
		float x;
		float y;
		float width;
		float height;
		int lightX; 
		int lightY;
		bool isLitUp;
	};
	struct Level
	{
		int floorX; 
		int floorWidth; 
		int floorY; 
		int floorHeight; 

		
		bool drawLaser;
		
	};

	public:
	Game( HWND hWnd,const KeyboardServer& kServer,const MouseServer& mServer );
	~Game();
	void Go();
private:
	void ComposeFrame();
	/********************************/
	/*  User Functions              */
		void UpdatePlayer();
		void UpdateEnemy();
		void UpdateBullet();
		void UpdateCamera();
		
		void drawBackground(int cameraX, int cameraY);
		void DrawTeleporter();
		void DrawPlayer(int x, int y);
		void DrawMenu();
		void DrawHelpMenu();
		
		void boxCoord();
		void enemyCoord();
		void theJump();
		
		void level1();
		void Part1();
		void Part1GFX();
	/********************************/
private:
	D3DGraphics gfx;
	KeyboardClient kbd;
	MouseClient mouse;
	DSound audio;
	/********************************/
	/*  User Variables              */
	int side[NENEMYS]; //for enemy movement
	int cameraX; //camera X coord
	int cameraY; //camera Y coord
	
	int cx; //teleporter X coord
	int cy; //teleporter Y coord
	int ParticleX[5]; //teleporter FX X coord
	int ParticleY[5];// teleporter FX Y coord
	int rad; // teleporter radius
	int g;
	int b;
	int MX;
	int MY;
	bool isPlayClicked;
	bool isHelpClicked;

	Menu menu[2];
	Player player;
	Enemy enemy[NENEMYS];
	Button button[NBUTTONS];
	Platform platform[NPLATFORMS];
	Level level;
	Sprite platformSpriteL;
	Sprite platformSpriteMid;
	Sprite platformSpriteR;
	Sprite platformRock;
	Sprite backgroundSprite;

	D3DCOLOR fontSurf[ 256 * 45 ];
	D3DCOLOR fontSurf1[ 512 * 84];
	Font fixedSys;
	Font fixedSys1;
	
/********************************/
};